"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var os = require('os');
var dgram = require('dgram');
var Netmask = require('netmask').Netmask;
var PREFIX = 'ION_DP';
var PORT = 41234;
var Publisher = (function () {
    function Publisher(name, port, interval) {
        if (interval === void 0) { interval = 2000; }
        this.name = name;
        this.port = port;
        this.interval = interval;
        this.running = false;
        if (name.indexOf(':') >= 0) {
            throw new Error('name can not contain ":"');
        }
        this.id = Math.round(Math.random() * 1000000);
    }
    Publisher.prototype.start = function () {
        var _this = this;
        if (this.running) {
            return;
        }
        this.running = true;
        var client = this.client = dgram.createSocket("udp4");
        client.on('listening', function () {
            client.setBroadcast(true);
            _this.timer = setInterval(_this.sayHello.bind(_this), _this.interval);
        });
        client.bind();
    };
    Publisher.prototype.stop = function () {
        if (!this.running) {
            return;
        }
        this.running = false;
        clearInterval(this.timer);
        this.timer = null;
        this.client.close();
        this.client = null;
    };
    Publisher.prototype.getMessage = function (iface) {
        var now = Date.now();
        var message = {
            t: now,
            id: this.id + '',
            name: this.name,
            host: os.hostname(),
            ip: iface.address,
            port: this.port
        };
        return PREFIX + JSON.stringify(message);
    };
    Publisher.prototype.sayHello = function () {
        for (var _i = 0, _a = this.getInterfaces(); _i < _a.length; _i++) {
            var iface = _a[_i];
            var message = new Buffer(this.getMessage(iface));
            this.client.send(message, 0, message.length, PORT, iface.broadcast);
        }
    };
    Publisher.prototype.getInterfaces = function () {
        var ips = [];
        var interfaces = os.networkInterfaces();
        return Object.keys(interfaces)
            .map(function (key) { return interfaces[key].find(function (i) { return i.family === 'IPv4'; }); })
            .filter(function (iface) { return !!iface; })
            .map(function (iface) {
            var i = Object.assign({}, iface);
            i.broadcast = computeMulticast(iface);
            return i;
        });
    };
    return Publisher;
}());
exports.Publisher = Publisher;
function computeMulticast(iface) {
    var ip = iface.address + '/' + iface.netmask;
    var block = new Netmask(ip);
    return block.broadcast;
}
